﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using libMpSDK;

namespace MP_SDK_Demo
{
    public partial class frmUpdateFirmware_Hardware : Form
    {
       

        public System.IO.Ports.SerialPort spModem;

        public String g_strFlilName = "";

        public frmUpdateFirmware_Hardware()
        {
            InitializeComponent();
        }

        private void frmUpdateFirmware_Shown(object sender, EventArgs e)
        {
            this.Refresh();
            Byte[] Transmit = new Byte[256];
            UInt32 StartMemAddress = 0x08000000;
            UInt32 StartInx = 0;
            Byte ACK = 0;

            spModem.BaudRate = 115200;
            spModem.Parity = System.IO.Ports.Parity.Even;
            spModem.DataBits = 8;

            labStatus.Text = "Updating...";
            labStatus.Refresh();

            // 取得 Byte 
            Byte[] byarrFile = System.IO.File.ReadAllBytes(g_strFlilName);

            // 取得總數
            Int32 i32DataLen = byarrFile.Length;

            try
            {
                // Check is in hardware boot
                Transmit[0] = 0x7F;
                spModem.Write(Transmit, 0, 1);
                ACK = (Byte)spModem.ReadByte();

                if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                // Check is in hardware boot
                Transmit[0] = 0x43;
                Transmit[1] = 0xBC;
                spModem.Write(Transmit, 0, 2);
                ACK = (Byte)spModem.ReadByte();

                if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                // Check is in hardware boot
                Transmit[0] = 0xFF;
                Transmit[1] = 0x00;
                spModem.Write(Transmit, 0, 2);
                ACK = (Byte)spModem.ReadByte();

                if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }
            }
            catch
            {
                MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK);
                this.Close();
                return;
            }


            pgStatus.Maximum = i32DataLen;
            pgStatus.Value = 0;

            try
            {
                while (i32DataLen > 0)
                {
                    // Send Write Mememory Command
                    Transmit[0] = 0x31;
                    Transmit[1] = 0xCE;
                    spModem.Write(Transmit, 0, 2);
                    ACK = (Byte)spModem.ReadByte();

                    if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                    // Set Address
                    Transmit[0] = BitConverter.GetBytes(StartMemAddress)[3];
                    Transmit[1] = BitConverter.GetBytes(StartMemAddress)[2];
                    Transmit[2] = BitConverter.GetBytes(StartMemAddress)[1];
                    Transmit[3] = BitConverter.GetBytes(StartMemAddress)[0];
                    Transmit[4] = m_byarCRC(Transmit, 4);
                    spModem.Write(Transmit, 0, 5);
                    ACK = (Byte)spModem.ReadByte();

                    if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                    // Size
                    Transmit[0] = (i32DataLen >= 252) ? (Byte)252 : BitConverter.GetBytes(i32DataLen)[0];
                    pgStatus.Value += Transmit[0];

                    Transmit[0] -= 1;

                    // Total N+1
                    for (Int32 i32Lp = 0; i32Lp < Transmit[0]+1; i32Lp++)
                    {
                        Transmit[1 + i32Lp] = byarrFile[StartInx + i32Lp];
                    }

                    Byte crcData = m_byarCRC(Transmit, Transmit[0]+2);
                    Transmit[Transmit[0] + 2] = crcData;

                    spModem.Write(Transmit, 0, Transmit[0] + 3);
                    ACK = (Byte)spModem.ReadByte();

                    if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                    StartMemAddress += (UInt32)(Transmit[0] + 1);
                    StartInx += Transmit[0];
                    StartInx += 1;
                    i32DataLen -= 252;
                }

            }
            catch
            {
                MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK);
                this.Close();
                return;
            }

            try
            {
                // Jump to application
                Transmit[0] = 0x21;
                Transmit[1] = 0xDE;
                spModem.Write(Transmit, 0, 2);
                ACK = (Byte)spModem.ReadByte();

                if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

                // Jump to application
                Transmit[0] = 0x08;
                Transmit[1] = 0x00;
                Transmit[2] = 0x00;
                Transmit[3] = 0x00;
                Transmit[4] = 0x08;
                spModem.Write(Transmit, 0, 5);
                ACK = (Byte)spModem.ReadByte();

                if (ACK != 0x79) { MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK); this.Close(); return; }

            }
            catch
            {
                MessageBox.Show("Try again.", "Caption", MessageBoxButtons.OK);
                this.Close();
            }
            spModem.Close();

            spModem.Open();

            this.Close();

        }

        private Byte m_byarCRC(Byte[] byarrData,Int32 i32Size)
        {
            Byte crc = 0x00;

            for (Int32 i32Lp = 0; i32Lp < i32Size; i32Lp++) crc ^= byarrData[i32Lp];

            return crc;
        }
    }
}
